#!/bin/bash
RUNTIMES=100 #采集次数
DURATION=1 #采集时间间隔
# 将需要测试的ip地址写在这,下面为示例代码
# PING_IP=(71.14.16.200)
PING_IP=()

FLAG_PERF_NETWORK=1
NETWORK_PERF_DIR=""
GTIMEOUT=5  #秒
FILETIME=$(date +%Y%m%d%H%M%S)
FILENAME=network_${FILETIME}.txt

function task_init()
{
    mkdir -p log
    if [ $FLAG_PERF_NETWORK == 1 ]; then
        DIR="log_net"
        mkdir -p ${DIR}
        NETWORK_PERF_DIR=${DIR}
    fi
	
    return 0
}

function download_tools()
{
	tools=(nicstat netstat ethtool)
	for tool in ${tools[@]}
	do
		apt install $tool
		sleep 3
	done	
}

function task_finish()
{
    [ ! -z ${NETWORK_PERF_DIR} ] && [ -d ${NETWORK_PERF_DIR} ] && mv ${NETWORK_PERF_DIR} ./log/
    echo "task finish"
}

#可获取：网卡速率，网卡利用率，网卡饱和率。
#会有两个结果，第一个是统计值，第二个结果是实时值
function task_get_nicstat()
{
    echo "==nicstat=================" | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
    timeout $GTIMEOUT nicstat -M 1 2 | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
}

#接口丢包, 网络连接数
function task_get_netstat()
{
    echo "==netstat_i=================" | tee -a ${NETWORK_PERF_DIR}/${FILENAME} #接口丢包
    timeout $GTIMEOUT netstat -i | tee -a ${NETWORK_PERF_DIR}/${FILENAME}

    echo "==netstat_anl=================" | tee -a ${NETWORK_PERF_DIR}/${FILENAME} #网络连接数
    netstat -antl | wc -l | tee -a ${NETWORK_PERF_DIR}/${FILENAME} #tcp
    netstat -anul | wc -l | tee -a ${NETWORK_PERF_DIR}/${FILENAME} #udp
    netstat -anxl | wc -l | tee -a ${NETWORK_PERF_DIR}/${FILENAME} #unix
}

#队列深度
function task_get_quelen()
{
    echo "==ethtool=================" | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
    for i in /sys/class/net/*; do
		if [[ ${i} =~ "veth" ]]; then
			continue			
		fi
		echo "${i##*/}:" | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
		ethtool -g "${i##*/}" | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
	done 
}

#时延
function task_get_latency()
{
    echo "==ping=================" | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
    for ip in ${PING_IP[@]}; do echo "${ip}:"; ping ${ip} -c 3; done | tee -a ${NETWORK_PERF_DIR}/${FILENAME}
}

function do_main()
{
    task_init
	download_tools
    [ -z ${NETWORK_PERF_DIR} ] || [ -d ${NETWORK_PERF_DIR} ] || echo "DIR not exist:${NETWORK_PERF_DIR}"
    [ -z ${NETWORK_PERF_DIR} ] && echo "DIR Empty:${NETWORK_PERF_DIR}" && return 1

    for i in $(seq 1 ${RUNTIMES})
    do
        task_get_latency
        task_get_nicstat
        task_get_netstat
        # task_get_quelen
        sleep ${DURATION}
    done

    task_finish
}

echo "result file: ${NETWORK_PERF_DIR}/${FILENAME}"
do_main